@@ -14,22 +14,47 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
+import contextlib
 import copy
 import re
 
 from genshi.builder import tag
 
 from trac.cache import cached
-from trac.config import *
+from trac.config import (
+    BoolOption, ConfigSection, ListOption, Option, OrderedExtensionsOption
+)
 from trac.core import *
 from trac.perm import IPermissionRequestor, PermissionCache, PermissionSystem
 from trac.resource import IResourceManager
 from trac.util import Ranges, as_int
 from trac.util.text import shorten_line
-from trac.util.translation import _, N_, gettext
+from trac.util.translation import _, N_, deactivate, gettext, reactivate
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
 
+class TicketFieldList(list):
+    """Improved ticket field list, allowing access by name."""
+    __slots__ = ['_map']
+
+    def __init__(self, *args):
+        super(TicketFieldList, self).__init__(*args)
+        self._map = dict((value['name'], value) for value in self)
+
+    def append(self, value):
+        super(TicketFieldList, self).append(value)
+        self._map[value['name']] = value
+
+    def by_name(self, name, default=None):
+        return self._map.get(name, default)
+
+    def __copy__(self):
+        return TicketFieldList(self)
+
+    def __deepcopy__(self, memo):
+        return TicketFieldList(copy.deepcopy(value, memo) for value in self)
+
+
 class ITicketActionController(Interface):
     """Extension point interface for components willing to participate
     in the ticket workflow.
@@ -173,6 +198,8 @@ class TicketSystem(Component):
     change_listeners = ExtensionPoint(ITicketChangeListener)
     milestone_change_listeners = ExtensionPoint(IMilestoneChangeListener)
 
+    realm = 'ticket'
+
     ticket_custom_section = ConfigSection('ticket-custom',
         """In this section, you can define additional fields for tickets. See
         TracTicketsCustomFields for more details.""")
@@ -180,8 +207,8 @@ class TicketSystem(Component):
     action_controllers = OrderedExtensionsOption('ticket', 'workflow',
         ITicketActionController, default='ConfigurableTicketWorkflow',
         include_missing=False,
-        doc="""Ordered list of workflow controllers to use for ticket actions
-            (''since 0.11'').""")
+        doc="""Ordered list of workflow controllers to use for ticket actions.
+            """)
 
     restrict_owner = BoolOption('ticket', 'restrict_owner', 'false',
         """Make the owner field of tickets use a drop-down menu.
@@ -192,13 +219,13 @@ class TicketSystem(Component):
         Please note that e-mail addresses are '''not''' obfuscated in the
         resulting drop-down menu, so this option should not be used if
         e-mail addresses must remain protected.
-        (''since 0.9'')""")
+        """)
 
     default_version = Option('ticket', 'default_version', '',
         """Default version for newly created tickets.""")
 
     default_type = Option('ticket', 'default_type', 'defect',
-        """Default type for newly created tickets (''since 0.9'').""")
+        """Default type for newly created tickets.""")
 
     default_priority = Option('ticket', 'default_priority', 'major',
         """Default priority for newly created tickets.""")
@@ -228,8 +255,12 @@ class TicketSystem(Component):
         """Default cc: list for newly created tickets.""")
 
     default_resolution = Option('ticket', 'default_resolution', 'fixed',
-        """Default resolution for resolving (closing) tickets
-        (''since 0.11'').""")
+        """Default resolution for resolving (closing) tickets.""")
+
+    allowed_empty_fields = ListOption('ticket', 'allowed_empty_fields',
+        'milestone, version', doc=
+        """Comma-separated list of `select` fields that can have
+        an empty value. (//since 1.1.2//)""")
 
     def __init__(self):
         self.log.debug('action controllers for ticket workflow: %r',
@@ -286,11 +317,11 @@ class TicketSystem(Component):
         del self.fields
 
     @cached
-    def fields(self, db):
+    def fields(self):
         """Return the list of fields available for tickets."""
         from trac.ticket import model
 
-        fields = []
+        fields = TicketFieldList()
 
         # Basic text fields
         fields.append({'name': 'summary', 'type': 'text',
@@ -305,7 +336,7 @@ class TicketSystem(Component):
 
         # Description
         fields.append({'name': 'description', 'type': 'textarea',
-                       'label': N_('Description')})
+                       'format': 'wiki', 'label': N_('Description')})
 
         # Default select and radio fields
         selects = [('type', N_('Type'), model.Type),
@@ -317,7 +348,7 @@ class TicketSystem(Component):
                    ('severity', N_('Severity'), model.Severity),
                    ('resolution', N_('Resolution'), model.Resolution)]
         for name, label, cls in selects:
-            options = [val.name for val in cls.select(self.env, db=db)]
+            options = [val.name for val in cls.select(self.env)]
             if not options:
                 # Fields without possible values are treated as if they didn't
                 # exist
@@ -328,7 +359,7 @@ class TicketSystem(Component):
             if name in ('status', 'resolution'):
                 field['type'] = 'radio'
                 field['optional'] = True
-            elif name in ('milestone', 'version'):
+            elif name in self.allowed_empty_fields:
                 field['optional'] = True
             fields.append(field)
 
@@ -340,11 +371,11 @@ class TicketSystem(Component):
 
         # Date/time fields
         fields.append({'name': 'time', 'type': 'time',
-                       'label': N_('Created')})
+                       'format': 'relative', 'label': N_('Created')})
         fields.append({'name': 'changetime', 'type': 'time',
-                       'label': N_('Modified')})
+                       'format': 'relative', 'label': N_('Modified')})
 
-        for field in self.get_custom_fields():
+        for field in self.custom_fields:
             if field['name'] in [f['name'] for f in fields]:
                 self.log.warning('Duplicate field name "%s" (ignoring)',
                                  field['name'])
@@ -357,7 +388,6 @@ class TicketSystem(Component):
                 self.log.warning('Invalid name for custom field: "%s" '
                                  '(ignoring)', field['name'])
                 continue
-            field['custom'] = True
             fields.append(field)
 
         return fields
@@ -370,30 +400,35 @@ class TicketSystem(Component):
         return copy.deepcopy(self.custom_fields)
 
     @cached
-    def custom_fields(self, db):
+    def custom_fields(self):
         """Return the list of custom ticket fields available for tickets."""
-        fields = []
+        fields = TicketFieldList()
         config = self.ticket_custom_section
         for name in [option for option, value in config.options()
                      if '.' not in option]:
             field = {
                 'name': name,
+                'custom': True,
                 'type': config.get(name),
                 'order': config.getint(name + '.order', 0),
-                'label': config.get(name + '.label') or name.capitalize(),
+                'label': config.get(name + '.label') or
+                         name.replace("_", " ").strip().capitalize(),
                 'value': config.get(name + '.value', '')
             }
             if field['type'] == 'select' or field['type'] == 'radio':
                 field['options'] = config.getlist(name + '.options', sep='|')
-                if '' in field['options']:
+                if '' in field['options'] or \
+                        field['name'] in self.allowed_empty_fields:
                     field['optional'] = True
-                    field['options'].remove('')
+                    if '' in field['options']:
+                        field['options'].remove('')
             elif field['type'] == 'text':
                 field['format'] = config.get(name + '.format', 'plain')
             elif field['type'] == 'textarea':
                 field['format'] = config.get(name + '.format', 'plain')
-                field['width'] = config.getint(name + '.cols')
                 field['height'] = config.getint(name + '.rows')
+            elif field['type'] == 'time':
+                field['format'] = config.get(name + '.format', 'datetime')
             fields.append(field)
 
         fields.sort(lambda x, y: cmp((x['order'], x['name']),
@@ -412,9 +447,7 @@ class TicketSystem(Component):
         """
         if self.restrict_owner:
             field['type'] = 'select'
-            allowed_owners = self.get_allowed_owners(ticket)
-            allowed_owners.insert(0, '< default >')
-            field['options'] = allowed_owners
+            field['options'] = self.get_allowed_owners(ticket)
             field['optional'] = True
 
     def get_allowed_owners(self, ticket=None):
@@ -443,18 +476,18 @@ class TicketSystem(Component):
     def get_permission_actions(self):
         return ['TICKET_APPEND', 'TICKET_CREATE', 'TICKET_CHGPROP',
                 'TICKET_VIEW', 'TICKET_EDIT_CC', 'TICKET_EDIT_DESCRIPTION',
-                'TICKET_EDIT_COMMENT', 'TICKET_BATCH_MODIFY',
+                'TICKET_EDIT_COMMENT',
                 ('TICKET_MODIFY', ['TICKET_APPEND', 'TICKET_CHGPROP']),
                 ('TICKET_ADMIN', ['TICKET_CREATE', 'TICKET_MODIFY',
                                   'TICKET_VIEW', 'TICKET_EDIT_CC',
                                   'TICKET_EDIT_DESCRIPTION',
-                                  'TICKET_EDIT_COMMENT',
-                                  'TICKET_BATCH_MODIFY'])]
+                                  'TICKET_EDIT_COMMENT'])]
 
     # IWikiSyntaxProvider methods
 
     def get_link_resolvers(self):
         return [('bug', self._format_link),
+                ('issue', self._format_link),
                 ('ticket', self._format_link),
                 ('comment', self._format_comment_link)]
 
@@ -477,7 +510,7 @@ class TicketSystem(Component):
             r = Ranges(link)
             if len(r) == 1:
                 num = r.a
-                ticket = formatter.resource('ticket', num)
+                ticket = formatter.resource(self.realm, num)
                 from trac.ticket.model import Ticket
                 if Ticket.id_is_valid(num) and \
                         'TICKET_VIEW' in formatter.perm(ticket):
@@ -488,8 +521,9 @@ class TicketSystem(Component):
                             SELECT type, summary, status, resolution
                             FROM ticket WHERE id=%s
                             """, (str(num),)):
-                        title = self.format_summary(summary, status,
-                                                    resolution, type)
+                        description = self.format_summary(summary, status,
+                                                          resolution, type)
+                        title = '#%s: %s' % (num, description)
                         href = formatter.href.ticket(num) + params + fragment
                         return tag.a(label, title=title, href=href,
                                      class_='%s ticket' % status)
@@ -515,13 +549,15 @@ class TicketSystem(Component):
                 if cnum != 'description' and cnum and not cnum[0].isdigit():
                     realm, id, cnum = elts # support old comment: style
                 id = as_int(id, None)
+                if realm in ('bug', 'issue'):
+                    realm = 'ticket'
                 resource = formatter.resource(realm, id)
         else:
             resource = formatter.resource
             cnum = target
 
-        if resource and resource.id and resource.realm == 'ticket' and \
-                cnum and (all(c.isdigit() for c in cnum) or cnum == 'description'):
+        if resource and resource.id and resource.realm == self.realm and \
+                cnum and (cnum.isdigit() or cnum == 'description'):
             href = title = class_ = None
             if self.resource_exists(resource):
                 from trac.ticket.model import Ticket
@@ -533,12 +569,17 @@ class TicketSystem(Component):
                     href = formatter.href.ticket(resource.id) + \
                            "#comment:%s" % cnum
                     if resource.id != formatter.resource.id:
+                        summary = self.format_summary(ticket['summary'],
+                                                      ticket['status'],
+                                                      ticket['resolution'],
+                                                      ticket['type'])
                         if cnum == 'description':
-                            title = _("Description for Ticket #%(id)s",
-                                      id=resource.id)
+                            title = _("Description for #%(id)s: %(summary)s",
+                                      id=resource.id, summary=summary)
                         else:
-                            title = _("Comment %(cnum)s for Ticket #%(id)s",
-                                      cnum=cnum, id=resource.id)
+                            title = _("Comment %(cnum)s for #%(id)s: "
+                                      "%(summary)s", cnum=cnum,
+                                      id=resource.id, summary=summary)
                         class_ = ticket['status'] + ' ticket'
                     else:
                         title = _("Description") if cnum == 'description' \
@@ -557,7 +598,7 @@ class TicketSystem(Component):
     # IResourceManager methods
 
     def get_resource_realms(self):
-        yield 'ticket'
+        yield self.realm
 
     def get_resource_description(self, resource, format=None, context=None,
                                  **kwargs):
@@ -598,13 +639,31 @@ class TicketSystem(Component):
         >>> resource_exists(env, t.resource)
         True
         """
-        if self.env.db_query("SELECT id FROM ticket WHERE id=%s",
-                             (resource.id,)):
+        try:
+            id_ = int(resource.id)
+        except (TypeError, ValueError):
+            return False
+        if self.env.db_query("SELECT id FROM ticket WHERE id=%s", (id_,)):
             if resource.version is None:
                 return True
             revcount = self.env.db_query("""
                 SELECT count(DISTINCT time) FROM ticket_change WHERE ticket=%s
-                """, (resource.id,))
+                """, (id_,))
             return revcount[0][0] >= resource.version
         else:
             return False
+
+
+@contextlib.contextmanager
+def translation_deactivated(ticket=None):
+    t = deactivate()
+    if ticket is not None:
+        ts = TicketSystem(ticket.env)
+        translated_fields = ticket.fields
+        ticket.fields = ts.get_ticket_fields()
+    try:
+        yield
+    finally:
+        if ticket is not None:
+            ticket.fields = translated_fields
+        reactivate(t)
