@@ -118,9 +118,6 @@ class TicketModule(Component):
 
     ticket_path_re = re.compile(r'/ticket/([0-9]+)$')
 
-    def __init__(self):
-        self._warn_for_default_attr = set()
-
     @lazy
     def must_preserve_newlines(self):
         preserve_newlines = self.preserve_newlines
@@ -612,8 +609,10 @@ class TicketModule(Component):
                 comment = req.args.get('edited_comment', '')
                 cnum = int(req.args['cnum_edit'])
                 change = ticket.get_change(cnum)
-                if not (req.authname and req.authname != 'anonymous'
-                        and change and change['author'] == req.authname):
+                if not change:
+                    raise TracError(_('Comment %(num)s not found', num=cnum))
+                if not (req.authname and req.authname != 'anonymous' and
+                        change['author'] == req.authname):
                     req.perm(ticket.resource).require('TICKET_EDIT_COMMENT')
                 ticket.modify_comment(change['date'], req.authname, comment)
                 req.redirect(req.href.ticket(ticket.id) + '#comment:%d' % cnum)
@@ -788,32 +787,23 @@ class TicketModule(Component):
     def _toggle_cc(self, req, cc):
         """Return an (action, recipient) tuple corresponding to a change
         of CC status for this user relative to the current `cc_list`."""
-        entries = []
-        email = req.session.get('email', '').strip()
-        if email:
-            entries.append(email)
+        entry = None
         if req.authname != 'anonymous':
-            entries.append(req.authname)
+            entry = req.authname
         else:
-            author = get_reporter_id(req, 'author').strip()
-            if author and author != 'anonymous':
-                email = author.split()[-1]
-                if (email[0], email[-1]) == ('<', '>'):
-                    email = email[1:-1]
-                entries.append(email)
-        add = []
-        remove = []
-        cc_list = self._cc_list(cc)
-        for entry in entries:
-            if entry in cc_list:
-                remove.append(entry)
+            email = req.session.get('email', '').strip()
+            if email:
+                entry = email
             else:
-                add.append(entry)
-        action = entry = None
-        if remove:
-            action, entry = ('remove', remove[0])
-        elif add:
-            action, entry = ('add', add[0])
+                author = get_reporter_id(req, 'author').strip()
+                if author and author != 'anonymous':
+                    email = author.split()[-1]
+                    if email.startswith('<') and email.endswith('>'):
+                        entry = email.strip('<>')
+        action = None
+        cc_list = self._cc_list(cc)
+        if entry:
+            action = 'remove' if entry in cc_list else 'add'
         return action, entry, cc_list
 
     def _populate(self, req, ticket, plain_fields=False):
@@ -1286,11 +1276,9 @@ class TicketModule(Component):
                 ticket.populate(ticket._old)
 
         comment = req.args.get('comment')
-        if comment:
-            if not ('TICKET_CHGPROP' in req.perm(resource) or
-                    'TICKET_APPEND' in req.perm(resource)):
-                add_warning(req, _("No permissions to add a comment."))
-                valid = False
+        if comment and 'TICKET_APPEND' not in req.perm(resource):
+            add_warning(req, _("No permissions to add a comment."))
+            valid = False
 
         # Mid air collision?
         if ticket.exists and (ticket._old or comment or force_collision_check):
@@ -1399,9 +1387,10 @@ class TicketModule(Component):
                            controller.__class__.__name__)
             controller.apply_action_side_effects(req, ticket, action)
 
-        # Notify
+        # Notify. Use author=req.authname rather than ticket['reporter']
+        # because ticket may be created on behalf of another user (#11949).
         event = TicketChangeEvent('created', ticket, ticket['time'],
-                                  ticket['reporter'])
+                                  req.authname)
         try:
             NotificationSystem(self.env).notify(event)
         except Exception as e:
