@@ -16,34 +16,42 @@
 tasks such as grouping or pagination.
 """
 
+from json import dumps
 from math import ceil
 import re
 
-__all__ = ['classes', 'first_last', 'group', 'istext', 'prepared_paginate', 
-           'paginate', 'Paginator']
+from trac.core import TracError
 
+__all__ = ['captioned_button', 'classes', 'first_last', 'group', 'istext',
+           'prepared_paginate', 'paginate', 'Paginator']
+__no_apidoc__ = 'prepared_paginate'
+
+def captioned_button(req, symbol, text):
+    """Return symbol and text or only symbol, according to user preferences."""
+    return symbol if req.session.get('ui.use_symbols') \
+        else u'%s %s' % (symbol, text)
 
 def classes(*args, **kwargs):
     """Helper function for dynamically assembling a list of CSS class names
     in templates.
-    
+
     Any positional arguments are added to the list of class names. All
     positional arguments must be strings:
-    
+
     >>> classes('foo', 'bar')
     u'foo bar'
-    
+
     In addition, the names of any supplied keyword arguments are added if they
     have a truth value:
-    
+
     >>> classes('foo', bar=True)
     u'foo bar'
     >>> classes('foo', bar=False)
     u'foo'
-    
+
     If none of the arguments are added to the list, this function returns
     `None`:
-    
+
     >>> classes(bar=False)
     """
     classes = list(filter(None, args)) + [k for k, v in kwargs.items() if v]
@@ -52,37 +60,40 @@ def classes(*args, **kwargs):
     return u' '.join(classes)
 
 def first_last(idx, seq):
+    """Generate ``first`` or ``last`` or both, according to the
+    position `idx` in sequence `seq`.
+    """
     return classes(first=idx == 0, last=idx == len(seq) - 1)
 
 
 def group(iterable, num, predicate=None):
     """Combines the elements produced by the given iterable so that every `n`
     items are returned as a tuple.
-    
+
     >>> items = [1, 2, 3, 4]
     >>> for item in group(items, 2):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3, 4)
-    
+
     The last tuple is padded with `None` values if its' length is smaller than
     `num`.
-    
+
     >>> items = [1, 2, 3, 4, 5]
     >>> for item in group(items, 2):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3, 4)
     (5, None)
-    
+
     The optional `predicate` parameter can be used to flag elements that should
     not be packed together with other items. Only those elements where the
     predicate function returns True are grouped with other elements, otherwise
     they are returned as a tuple of length 1:
-    
+
     >>> items = [1, 2, 3, 4]
     >>> for item in group(items, 2, lambda x: x != 3):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3,)
     (4, None)
@@ -104,6 +115,7 @@ def group(iterable, num, predicate=None):
 
 
 def istext(text):
+    """`True` for text (`unicode` and `str`), but `False` for `Markup`."""
     from genshi.core import Markup
     return isinstance(text, basestring) and not isinstance(text, Markup)
 
@@ -116,14 +128,14 @@ def prepared_paginate(items, num_items, max_per_page):
 
 def paginate(items, page=0, max_per_page=10):
     """Simple generic pagination.
-    
+
     Given an iterable, this function returns:
      * the slice of objects on the requested page,
      * the total number of items, and
      * the total number of pages.
-    
+
     The `items` parameter can be a list, tuple, or iterator:
-    
+
     >>> items = range(12)
     >>> items
     [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
@@ -135,9 +147,9 @@ def paginate(items, page=0, max_per_page=10):
     ([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], 12, 2)
     >>> paginate(iter(items), page=1)
     ([10, 11], 12, 2)
-    
+
     This function also works with generators:
-    
+
     >>> def generate():
     ...     for idx in range(12):
     ...         yield idx
@@ -145,15 +157,18 @@ def paginate(items, page=0, max_per_page=10):
     ([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], 12, 2)
     >>> paginate(generate(), page=1)
     ([10, 11], 12, 2)
-    
+
     The `max_per_page` parameter can be used to set the number of items that
     should be displayed per page:
-    
+
     >>> items = range(12)
     >>> paginate(items, page=0, max_per_page=6)
     ([0, 1, 2, 3, 4, 5], 12, 2)
     >>> paginate(items, page=1, max_per_page=6)
     ([6, 7, 8, 9, 10, 11], 12, 2)
+
+    :raises TracError: if `page` is out of the range of the paginated
+                       results.
     """
     if not page:
         page = 0
@@ -163,8 +178,9 @@ def paginate(items, page=0, max_per_page=10):
     count = None
     if hasattr(items, '__len__'):
         count = len(items)
-        if count:
-            assert start < count, 'Page %d out of range' % page
+        if count and start >= count:
+            from trac.util.translation import _
+            raise TracError(_("Page %(page)s is out of range.", page=page))
 
     try: # Try slicing first for better performance
         retval = items[start:stop]
@@ -186,6 +202,7 @@ def paginate(items, page=0, max_per_page=10):
 
 
 class Paginator(object):
+    """Pagination controller"""
 
     def __init__(self, items, page=0, max_per_page=10, num_items=None):
         if not page:
@@ -196,7 +213,7 @@ class Paginator(object):
         else:
             items, num_items, num_pages = prepared_paginate(items, num_items,
                                                             max_per_page)
-        offset = page * max_per_page        
+        offset = page * max_per_page
         self.page = page
         self.max_per_page = max_per_page
         self.items = items
@@ -217,20 +234,20 @@ class Paginator(object):
     def __setitem__(self, idx, value):
         self.items[idx] = value
 
+    @property
     def has_more_pages(self):
         return self.num_pages > 1
-    has_more_pages = property(has_more_pages)
 
+    @property
     def has_next_page(self):
         return self.page + 1 < self.num_pages
-    has_next_page = property(has_next_page)
 
+    @property
     def has_previous_page(self):
         return self.page > 0
-    has_previous_page = property(has_previous_page)
-   
+
     def get_shown_pages(self, page_index_count = 11):
-        if self.has_more_pages == False:
+        if not self.has_more_pages:
             return range(1, 2)
 
         min_page = 1
@@ -251,17 +268,18 @@ class Paginator(object):
         from trac.util.translation import _
         start, stop = self.span
         total = self.num_items
-        if start+1 == stop:
+        if start + 1 == stop:
             return _("%(last)d of %(total)d", last=stop, total=total)
         else:
             return _("%(start)d - %(stop)d of %(total)d",
-                    start=self.span[0]+1, stop=self.span[1], total=total)
+                    start=self.span[0] + 1, stop=self.span[1], total=total)
 
 
-def separated(items, sep=','):
+def separated(items, sep=',', last=None):
     """Yield `(item, sep)` tuples, one for each element in `items`.
 
-    `sep` will be `None` for the last item.
+    The separator after the last item is specified by the `last` parameter,
+    which defaults to `None`. (Since 1.1.3)
 
     >>> list(separated([1, 2]))
     [(1, ','), (2, None)]
@@ -269,51 +287,27 @@ def separated(items, sep=','):
     >>> list(separated([1]))
     [(1, None)]
 
-    >>> list(separated("abc", ':'))
+    >>> list(separated('abc', ':'))
     [('a', ':'), ('b', ':'), ('c', None)]
+
+    >>> list(separated((1, 2, 3), sep=';', last='.'))
+    [(1, ';'), (2, ';'), (3, '.')]
     """
     items = iter(items)
-    last = items.next()
+    next = items.next()
     for i in items:
-        yield last, sep
-        last = i
-    yield last, None
+        yield next, sep
+        next = i
+    yield next, last
 
 
-try:
-    from json import dumps
+_js_quote = dict((c, '\\u%04x' % ord(c)) for c in '&<>')
+_js_quote_re = re.compile('[' + ''.join(_js_quote) + ']')
 
-    _js_quote = dict((c, '\\u%04x' % ord(c)) for c in '&<>')
-    _js_quote_re = re.compile('[' + ''.join(_js_quote) + ']')
-    
-    def to_json(value):
-        """Encode `value` to JSON."""
-        def replace(match):
-            return _js_quote[match.group(0)]
-        text = dumps(value, sort_keys=True, separators=(',', ':'))
-        return _js_quote_re.sub(replace, text)
 
-except ImportError:
-    from trac.util.text import javascript_quote
-    
-    def to_json(value):
-        """Encode `value` to JSON."""
-        if isinstance(value, basestring):
-            return '"%s"' % javascript_quote(value)
-        elif value is None:
-            return 'null'
-        elif value is False:
-            return 'false'
-        elif value is True:
-            return 'true'
-        elif isinstance(value, (int, long)):
-            return str(value)
-        elif isinstance(value, float):
-            return repr(value)
-        elif isinstance(value, (list, tuple)):
-            return '[%s]' % ','.join(to_json(each) for each in value)
-        elif isinstance(value, dict):
-            return '{%s}' % ','.join('%s:%s' % (to_json(k), to_json(v))
-                                     for k, v in sorted(value.iteritems()))
-        else:
-            raise TypeError('Cannot encode type %s' % value.__class__.__name__)
+def to_json(value):
+    """Encode `value` to JSON."""
+    def replace(match):
+        return _js_quote[match.group(0)]
+    text = dumps(value, sort_keys=True, separators=(',', ':'))
+    return _js_quote_re.sub(replace, text)
