@@ -19,11 +19,13 @@ import re
 from genshi.builder import tag
 
 from trac.util.concurrency import ThreadLocal, threading
+from trac.util.text import cleandoc
 
 
-__all__ = ['gettext', 'ngettext', 'gettext_noop', 'ngettext_noop', 
+__all__ = ['gettext', 'ngettext', 'gettext_noop', 'ngettext_noop',
            'tgettext', 'tgettext_noop', 'tngettext', 'tngettext_noop']
 
+
 def safefmt(string, kwargs):
     if kwargs:
         try:
@@ -40,9 +42,10 @@ def dgettext_noop(domain, string, **kwargs):
     return gettext_noop(string, **kwargs)
 
 N_ = _noop = lambda string: string
+cleandoc_ = cleandoc
 
 def ngettext_noop(singular, plural, num, **kwargs):
-    string = (plural, singular)[num == 1]
+    string = singular if num == 1 else plural
     kwargs.setdefault('num', num)
     return safefmt(string, kwargs)
 
@@ -57,13 +60,13 @@ def _tag_kwargs(trans, kwargs):
     return tag(*trans_elts)
 
 def tgettext_noop(string, **kwargs):
-    return kwargs and _tag_kwargs(string, kwargs) or string
+    return _tag_kwargs(string, kwargs) if kwargs else string
 
 def dtgettext_noop(domain, string, **kwargs):
     return tgettext_noop(string, **kwargs)
 
 def tngettext_noop(singular, plural, num, **kwargs):
-    string = (plural, singular)[num == 1]
+    string = singular if num == 1 else plural
     kwargs.setdefault('num', num)
     return _tag_kwargs(string, kwargs)
 
@@ -128,12 +131,9 @@ try:
         # Public API
 
         def add_domain(self, domain, env_path, locales_dir):
-            self._plugin_domains_lock.acquire()
-            try:
+            with self._plugin_domains_lock:
                 domains = self._plugin_domains.setdefault(env_path, {})
                 domains[domain] = locales_dir
-            finally:
-                self._plugin_domains_lock.release()
 
         def make_activable(self, get_locale, env_path=None):
             self._current.args = (get_locale, env_path)
@@ -145,28 +145,30 @@ try:
                 self._activate_failed = True
                 return
             t = Translations.load(locale_dir, locale or 'en_US')
-            if not t or t.__class__ is NullTranslations:
+            if not isinstance(t, Translations):
                 t = self._null_translations
-            elif env_path:
-                self._plugin_domains_lock.acquire()
-                try:
-                    domains = self._plugin_domains.get(env_path, {}).items()
-                finally:
-                    self._plugin_domains_lock.release()
-                for domain, dirname in domains:
-                    t.add(Translations.load(dirname, locale, domain))
+            else:
+                self._add(t, Translations.load(locale_dir, locale or 'en_US',
+                                               'tracini'))
+                if env_path:
+                    with self._plugin_domains_lock:
+                        domains = self._plugin_domains.get(env_path, {})
+                        domains = domains.items()
+                    for domain, dirname in domains:
+                        self._add(t, Translations.load(dirname, locale,
+                                                       domain))
             self._current.translations = t
             self._activate_failed = False
-         
+
         def deactivate(self):
             self._current.args = None
             t, self._current.translations = self._current.translations, None
             return t
-         
+
         def reactivate(self, t):
             if t:
                 self._current.translations = t
-    
+
         @property
         def active(self):
             return self._current.translations or self._null_translations
@@ -182,6 +184,12 @@ try:
             return self._current.translations is not None \
                    or self._activate_failed
 
+        # Internal methods
+
+        def _add(self, t, translations):
+            if isinstance(translations, Translations):
+                t.add(translations)
+
         # Delegated methods
 
         def __getattr__(self, name):
@@ -224,7 +232,7 @@ try:
         def tgettext(self, string, **kwargs):
             def _tgettext():
                 trans = self.active.ugettext(string)
-                return kwargs and _tag_kwargs(trans, kwargs) or trans
+                return _tag_kwargs(trans, kwargs) if kwargs else trans
             if not self.isactive:
                 return LazyProxy(_tgettext)
             return _tgettext()
@@ -232,7 +240,7 @@ try:
         def dtgettext(self, domain, string, **kwargs):
             def _dtgettext():
                 trans = self.active.dugettext(domain, string)
-                return kwargs and _tag_kwargs(trans, kwargs) or trans
+                return _tag_kwargs(trans, kwargs) if kwargs else trans
             if not self.isactive:
                 return LazyProxy(_dtgettext)
             return _dtgettext()
@@ -253,12 +261,12 @@ try:
                 trans = self.active.dungettext(domain, singular, plural, num)
                 if '%(num)' in trans:
                     kwargs.update(num=num)
-                return kwargs and _tag_kwargs(trans, kwargs) or trans
+                return _tag_kwargs(trans, kwargs) if kwargs else trans
             if not self.isactive:
                 return LazyProxy(_dtngettext)
             return _dtngettext()
 
-    
+
     translations = TranslationsProxy()
 
     def domain_functions(domain, *symbols):
@@ -267,7 +275,7 @@ try:
         :param domain: domain used for partial instantiation
         :param symbols: remaining parameters are the name of commonly used
                         translation function which will be bound to the domain
-                        
+
         Note: the symbols can also be given as an iterable in the 2nd argument.
         """
         if symbols and not isinstance(symbols[0], basestring):
@@ -288,18 +296,24 @@ try:
             return lambda *args, **kw: _functions[symbol](domain, *args, **kw)
         return [wrapdomain(s) for s in symbols]
 
-    gettext = translations.gettext 
-    _ = gettext 
-    dgettext = translations.dgettext 
-    ngettext = translations.ngettext 
-    dngettext = translations.dngettext 
-    tgettext = translations.tgettext 
-    tag_ = tgettext 
-    dtgettext = translations.dtgettext 
-    tngettext = translations.tngettext 
-    tagn_ = tngettext 
-    dtngettext = translations.dtngettext 
-    
+    gettext = translations.gettext
+    _ = gettext
+    dgettext = translations.dgettext
+    ngettext = translations.ngettext
+    dngettext = translations.dngettext
+    tgettext = translations.tgettext
+    tag_ = tgettext
+    dtgettext = translations.dtgettext
+    tngettext = translations.tngettext
+    tagn_ = tngettext
+    dtngettext = translations.dtngettext
+
+    def add_domain(domain, env_path, locale_dir):
+        translations.add_domain(domain, env_path, locale_dir)
+
+    def activate(locale, env_path=None):
+        translations.activate(locale, env_path)
+
     def deactivate():
         """Deactivate translations.
         :return: the current Translations, if any
@@ -319,12 +333,6 @@ try:
         """
         translations.make_activable(get_locale, env_path)
 
-    def activate(locale, env_path=None):
-        translations.activate(locale, env_path)
-
-    def add_domain(domain, env_path, locale_dir):
-        translations.add_domain(domain, env_path, locale_dir)
-
     def get_translations():
         return translations
 
@@ -333,21 +341,37 @@ try:
         translations are available.
         """
         try:
-            return [dirname for dirname
-                    in pkg_resources.resource_listdir('trac', 'locale')
-                    if '.' not in dirname]
+            locales = [dirname for dirname
+                       in pkg_resources.resource_listdir('trac', 'locale')
+                       if '.' not in dirname
+                       and pkg_resources.resource_exists(
+                        'trac', 'locale/%s/LC_MESSAGES/messages.mo' % dirname)]
+            return locales
         except Exception:
             return []
 
     def get_negotiated_locale(preferred_locales):
         def normalize(locale_ids):
-            return [id.replace('_', '-') for id in locale_ids if id]
-        return Locale.negotiate(normalize(preferred_locales),
-                                normalize(get_available_locales()), sep='-')
-        
+            return [id.replace('-', '_') for id in locale_ids if id]
+        available_locales = get_available_locales()
+        if 'en_US' not in available_locales:
+            available_locales.append('en_US')
+        locale = Locale.negotiate(normalize(preferred_locales),
+                                  normalize(available_locales))
+        if locale and str(locale) not in available_locales:
+            # The list of get_available_locales() must include locale
+            # identifier from str(locale), but zh_* don't be included after
+            # Babel 1.0. Avoid expanding zh_* to zh_Hans_CN and zh_Hant_TW
+            # to clear "script" property of Locale instance. See #11258.
+            locale._data  # load localedata before clear script property
+            locale.script = None
+            assert str(locale) in available_locales
+        return locale
+
     has_babel = True
 
 except ImportError: # fall back on 0.11 behavior, i18n functions are no-ops
+    Locale = None
     gettext = _ = gettext_noop
     dgettext = dgettext_noop
     ngettext = ngettext_noop
@@ -358,7 +382,7 @@ except ImportError: # fall back on 0.11 behavior, i18n functions are no-ops
     dtngettext = dtngettext_noop
 
     translations = NullTranslationsBabel()
-    
+
     def activate(locale, env_path=None):
         pass
 
@@ -377,5 +401,5 @@ except ImportError: # fall back on 0.11 behavior, i18n functions are no-ops
     def get_available_locales():
         return []
 
-    def get_negotiated_locale(preferred=None, default=None):
+    def get_negotiated_locale(preferred_locales):
         return None
