@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2006 Edgewall Software
+# Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -14,12 +14,30 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+from abc import ABCMeta, abstractmethod
+import errno
 import sys
 from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
 from SocketServer import ForkingMixIn, ThreadingMixIn
 import urllib
 
 
+# winsock errors
+_WSAECONNABORTED = 10053
+_WSAECONNRESET = 10054
+
+
+def is_client_disconnect_exception(e):
+    """Determines whether the exception was caused by a disconnecting client.
+
+    :type e: IOError
+    :rtype: bool
+    """
+    return e.errno in (errno.EPIPE, errno.ECONNRESET, # Unix
+                       _WSAECONNABORTED, _WSAECONNRESET, # Windows
+                       None) # mod_wsgi, uwsgi, ... (see #12650)
+
+
 class _ErrorsWrapper(object):
 
     def __init__(self, logfunc):
@@ -58,6 +76,8 @@ class _FileWrapper(object):
 class WSGIGateway(object):
     """Abstract base class for WSGI servers or gateways."""
 
+    __metaclass__ = ABCMeta
+
     wsgi_version = (1, 0)
     wsgi_multithread = True
     wsgi_multiprocess = True
@@ -70,6 +90,8 @@ class WSGIGateway(object):
         environ['wsgi.url_scheme'] = 'http'
         if environ.get('HTTPS', '').lower() in ('yes', 'on', '1'):
             environ['wsgi.url_scheme'] = 'https'
+        elif environ.get('HTTP_X_FORWARDED_PROTO', '').lower() == 'https':
+            environ['wsgi.url_scheme'] = 'https'
         environ['wsgi.input'] = stdin
         environ['wsgi.errors'] = stderr
         environ['wsgi.multithread'] = self.wsgi_multithread
@@ -81,20 +103,22 @@ class WSGIGateway(object):
 
         self.headers_set = []
         self.headers_sent = []
+        self.use_chunked = False
 
     def run(self, application):
         """Start the gateway with the given WSGI application."""
         response = application(self.environ, self._start_response)
         try:
-            if isinstance(response, self.wsgi_file_wrapper) \
+            if self.wsgi_file_wrapper is not None \
+                    and isinstance(response, self.wsgi_file_wrapper) \
                     and hasattr(self, '_sendfile'):
                 self._sendfile(response.fileobj)
             else:
                 for chunk in response:
                     if chunk:
                         self._write(chunk)
-                if not self.headers_sent:
-                    self._write('')
+                if not self.headers_sent or self.use_chunked:
+                    self._write('') # last chunk '\r\n0\r\n' if use_chunked
         finally:
             if hasattr(response, 'close'):
                 response.close()
@@ -113,19 +137,23 @@ class WSGIGateway(object):
         self.headers_set = [status, headers]
         return self._write
 
+    @abstractmethod
     def _write(self, data):
         """Callback for writing data to the response.
-        
+
         Concrete subclasses must implement this method."""
-        raise NotImplementedError
+        pass
 
 
 class WSGIRequestHandler(BaseHTTPRequestHandler):
 
     def setup_environ(self):
         self.raw_requestline = self.rfile.readline()
-        if not self.parse_request(): # An error code has been sent, just exit
+        if (self.rfile.closed or              # disconnect
+                not self.raw_requestline or   # empty request
+                not self.parse_request()):    # invalid request
             self.close_connection = 1
+            # note that in the latter case, an error code has already been sent
             return
 
         environ = self.server.environ.copy()
@@ -155,7 +183,7 @@ class WSGIRequestHandler(BaseHTTPRequestHandler):
 
         for name, value in [header.split(':', 1) for header
                             in self.headers.headers]:
-            name = name.replace('-', '_').upper();
+            name = name.replace('-', '_').upper()
             value = value.strip()
             if name in environ:
                 # skip content length, type, etc.
@@ -169,15 +197,30 @@ class WSGIRequestHandler(BaseHTTPRequestHandler):
         return environ
 
     def handle_one_request(self):
-        environ = self.setup_environ()
-        gateway = self.server.gateway(self, environ)
-        gateway.run(self.server.application)
+        try:
+            environ = self.setup_environ()
+        except IOError as e:
+            environ = None
+            if is_client_disconnect_exception(e):
+                self.close_connection = 1
+            else:
+                raise
+        if environ:
+            gateway = self.server.gateway(self, environ)
+            gateway.run(self.server.application)
+        # else we had no request or a bad request: we simply exit (#3043)
 
     def finish(self):
         """We need to help the garbage collector a little."""
-        BaseHTTPRequestHandler.finish(self)
-        self.wfile = None
-        self.rfile = None
+        try:
+            BaseHTTPRequestHandler.finish(self)
+        except IOError as e:
+            # ignore an exception if client disconnects
+            if not is_client_disconnect_exception(e):
+                raise
+        finally:
+            self.wfile = None
+            self.rfile = None
 
 
 class WSGIServerGateway(WSGIGateway):
@@ -189,14 +232,37 @@ class WSGIServerGateway(WSGIGateway):
 
     def _write(self, data):
         assert self.headers_set, 'Response not started'
+        if self.handler.wfile.closed:
+            return # don't write to an already closed file (fix for #1183)
 
-        if not self.headers_sent:
-            status, headers = self.headers_sent = self.headers_set
-            self.handler.send_response(int(status[:3]))
-            for name, value in headers:
-                self.handler.send_header(name, value)
-            self.handler.end_headers()
-        self.handler.wfile.write(data)
+        try:
+            if not self.headers_sent:
+                # Worry at the last minute about Content-Length. If not
+                # yet set, use either chunked encoding or close connection
+                status, headers = self.headers_sent = self.headers_set
+                if any(n.lower() == 'content-length' for n, v in headers):
+                    self.use_chunked = False
+                else:
+                    self.use_chunked = (
+                        self.environ['SERVER_PROTOCOL'] >= 'HTTP/1.1' and
+                        self.handler.protocol_version >= 'HTTP/1.1')
+                    if self.use_chunked:
+                        headers.append(('Transfer-Encoding', 'chunked'))
+                    else:
+                        headers.append(('Connection', 'close'))
+                self.handler.send_response(int(status[:3]))
+                for name, value in headers:
+                    self.handler.send_header(name, value)
+                self.handler.end_headers()
+            if self.use_chunked:
+                self.handler.wfile.write('%x\r\n%s\r\n' % (len(data), data))
+            else:
+                self.handler.wfile.write(data)
+        except IOError as e:
+            if is_client_disconnect_exception(e):
+                self.handler.close_connection = 1
+            else:
+                raise
 
 
 class WSGIServer(HTTPServer):
