@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2004-2013 Edgewall Software
@@ -84,12 +85,17 @@ history:imported only for summary, priority. closed date and owner fields
 severities:no field in source data
 """
 
+import optparse
+import sys
+import time
+from xml.etree.ElementTree import ElementTree
+
+from trac.env import Environment
+from trac.util.text import printerr
 
 #: rename users from SF to Trac
 user_map = {"nobody":"anonymous"}
 
-
-
 complete_msg = """
 Conversion complete.
 
@@ -97,13 +103,6 @@ You may want to login into Trac to verify names for ticket owners. You may
 also want to rename ticket types and priorities to default.
 """
 
-from __future__ import with_statement
-
-from xml.etree.ElementTree import ElementTree
-import time
-import sys
-
-import trac.env
 
 # --- utility
 class DBNotEmpty(Exception):
@@ -341,7 +340,8 @@ class ExportedProjectData(object):
 
             # tickets
             for tck in tr.tracker_items:
-                if type(tck) == str: print repr(tck)
+                if type(tck) == str:
+                    print(repr(tck))
                 self.tickets.append(tck)
                 if int(tck.priority) not in self.priorities:
                     self.priorities.append(int(tck.priority))
@@ -389,7 +389,7 @@ class ExportedProjectData(object):
 
 class TracDatabase(object):
     def __init__(self, path):
-        self.env = trac.env.Environment(path)
+        self.env = Environment(path)
 
     def hasTickets(self):
         return int(self.env.db_query("SELECT count(*) FROM ticket")[0][0]) > 0
@@ -528,24 +528,25 @@ def importData(f, env, opt):
     components = dict(project.get_categories(noid=True)).items()
     components.sort()
     components = [x for x in components if x[0] in used_cat_names]
-    print "%d out of %d categories are used and will be converted to the following components:\n  %s" \
-        % (len(components), len(project.get_categories()), components)
-    print "..renaming component owners:"
+    print("%d out of %d categories are used and will be converted to the"
+          " following components:\n  %s"
+          % (len(components), len(project.get_categories()), components))
+    print("..renaming component owners:")
     for i,c in enumerate(components):
         if c[1] in user_map:
             components[i] = (c[0], user_map[c[1]])
-    print "  %s" % components
+    print("  %s" % components)
 
-    print "%d groups which will be converted to the following versions:\n  %s" \
-        % (len(project.groups), project.groups)
-    print "%d resolutions found :\n  %s" \
-        % (len(project.resolutions), project.resolutions)
+    print("%d groups which will be converted to the following versions:\n"
+          "  %s" % (len(project.groups), project.groups))
+    print("%d resolutions found :\n  %s"
+          % (len(project.resolutions), project.resolutions))
     resolutions = [(k,project.used_resolutions[k])
                    for k in project.used_resolutions]
     resolutions.sort(key=lambda x:int(x[0]))
-    print ".. only %d used will be imported:\n  %s" \
-        % (len(resolutions), resolutions)
-    print "Priorities used so far: %s" % project.priorities
+    print(".. only %d used will be imported:\n  %s"
+          % (len(resolutions), resolutions))
+    print("Priorities used so far: %s" % project.priorities)
     if not(raw_input("Continue [y/N]?").lower() == 'y'):
         sys.exit()
 
@@ -586,7 +587,7 @@ def importData(f, env, opt):
                                description=t.details,
                                keywords='sf' + t.id)
 
-            print 'Imported %s as #%d' % (t.id, i)
+            print("Imported %s as #%d" % (t.id, i))
 
             if len(t.attachments):
                 attmsg = "SourceForge attachments:\n"
@@ -602,8 +603,8 @@ def importData(f, env, opt):
                                       time=time.strftime("%Y-%m-%d %H:%M:%S",
                                               time.localtime(int(t.submit_date))),
                                       author=None, value=attmsg)
-                print '    added information about %d attachments for #%d' % \
-                        (len(t.attachments), i)
+                print("    added information about %d attachments for #%d"
+                      % (len(t.attachments), i))
 
             for msg in t.followups:
                 """
@@ -619,7 +620,8 @@ def importData(f, env, opt):
                                       author=msg.submitter,
                                       value=msg.details)
             if t.followups:
-                print '    imported %d messages for #%d' % (len(t.followups), i)
+                print("    imported %d messages for #%d"
+                      % (len(t.followups), i))
 
             # Import history
             """
@@ -660,7 +662,7 @@ def importData(f, env, opt):
                 elif h.field_name in ("summary", "priority"):
                     f = h.field_name
                     oldvalue = h.old_value
-                    newvalue = revision.get(h.field_name, None)
+                    newvalue = revision.get(h.field_name)
                 elif h.field_name == 'assigned_to':
                     f = "owner"
                     newvalue = revision['assignee']
@@ -688,12 +690,11 @@ def importData(f, env, opt):
                 if h.field_name != 'assigned_to':
                     revision[h.field_name] = h.old_value
             if changes:
-                print '    processed %d out of %d history items for #%d' % \
-                        (changes, len(t.history_entries), i)
+                print("    processed %d out of %d history items for #%d"
+                      % (changes, len(t.history_entries), i))
 
 
 def main():
-    import optparse
     p = optparse.OptionParser(
             "Usage: %prog xml_export.xml /path/to/trac/environment")
     opt, args = p.parse_args()
@@ -702,11 +703,11 @@ def main():
 
     try:
         importData(open(args[0]), args[1], opt)
-    except DBNotEmpty, e:
-        print 'Error:', e
+    except DBNotEmpty as e:
+        printerr("Error: %s" % e)
         sys.exit(1)
 
-    print complete_msg
+    print(complete_msg)
 
 
 if __name__ == '__main__':
